<?php
/**
 * ATTENTION : UTF-8 (éàöù)
 * [20/04/2005] Version 1.0
 * [23/04/2005] Version 1.1
 * [25/04/2005] Version 1.2
 * Ajout de la rubrique RSS
 * 
 * [29/04/2005] Version 1.3
 * Ajout de la rubrique Calendar
 * 
 * [09/05/2005] Version 1.4
 * Ajout du module Contacts
 * 
 * [14/05/2005] Version 1.5
 * Ajout du nouveau système de getter/setter
 * 
 * [15/05/2005] Version 1.6
 * Ajout du module Members
 * 
 * @ignore
 */
/**
 * Apo
 * @author			neolao <neo@neolao.com>
 * @version			1.6 (15/05/2005)
 */
class Apo {
	// ----------------------------- CONSTANTES --------------------------------
	
	// ----------------------------- VARIABLES ---------------------------------
	/**
	 * La racine du site
	 * @var string
	 */
	private $_serverRoot;
	/**
	 * Le fichier d'index du site
	 * @var string
	 */
	private $_webIndex;
	/**
	 * La racine web de APO
	 * @var string
	 */
	private $_webRoot;
	/**
	 * L'instance de connexion à la base de données
	 * @var Connection
	 */
	private $_db;
	/**
	 * Le préfix des tables de la base de données
	 * @var string
	 */
	private $_dbPrefix;
	/**
	 * Indique si l'utilisateur est identifié
	 * @var bool
	 */
	private $_loged;
	/**
	 * Indique si l'utilisateur n'utilise pas les cookies
	 * @var bool
	 */
	private $_noCookie;
	/**
	 * Les informations sur l'utilisateur
	 * @var array
	 */
	private $_userInfos;
	/**
	 * La langue utilisée
	 * @var string
	 */
	private $_lang;
	/**
	 * Les textes du site
	 * @var array
	 */
	private $_texts;
	/**
	 * Le thème utilisée
	 * @var string
	 */
	private $_theme;
	/**
	 * Les informations de la page
	 * @var array
	 */
	private $_pageInfos;
	/**
	 * Message d'erreur
	 * @var string
	 */
	private $_error;
	/**
	 * Message
	 * @var string
	 */
	private $_message;
	/**
	 * URL du site de mise à jour
	 * @var string
	 */
	private $_updateUrl;
	/**
	 * Extension des fichiers php5
	 * @var string
	 */
	private $_php5Extension;
	// ----------------------------- EVENEMENTS --------------------------------
	
	/*===================== CONSTRUCTEUR & DESCTRUCTEUR ======================*/
	/*========================================================================*/
	/**
	 * Initialisation de la class
	 * @param string $dir La racine du site
	 * @param string $file Le fichier d'index
	 * @param Connection $db L'instance de connexion à la base de données
	 */
	function Apo($dir, $file, &$db){
		$this->_serverRoot = $dir;
		$this->_webIndex = $file;
		$this->_db = $db;
		
		// Initialisation
		$this->_dbPrefix = "apo_";
		$this->_loged = false;
		$this->_lang = "fr";
		$this->_theme = "default";
		$this->_userInfos = array();
		$this->_pageInfos = array();
	}
	/**
	 * Destruction de la class
	 */
	function __destruct(){
	
	}
	/*=============== FIN = CONSTRUCTEUR & DESCTRUCTEUR = FIN ================*/
	/*========================================================================*/
	
	/*=========================== METHODES PRIVEES ===========================*/
	/*========================================================================*/
	/**
	 * Importe une configuration à partir d'un fichier .ini
	 * @param string $path Le chemin du fichier de config
	 */
	private function setConfig($path){
		if(file_exists($path)){
			$o = parse_ini_file($path);
			
			if(!empty($o["db_prefix"])){
				$this->_dbPrefix = $o["db_prefix"];
			}
			if(!empty($o["web_root"])){
				$this->_webRoot = $o["web_root"];
			}
			if(!empty($o["update_url"])){
				$this->_updateUrl = $o["update_url"];
			}
			if(!empty($o["php5_extension"])){
				$this->_php5Extension = $o["php5_extension"];
			}
		}
	}
	/**
	 * Charge le fichier de langue
	 * @param string $lang La langue
	 */
	private function setLang($lang){
		if(file_exists($this->_serverRoot."/lang/".$lang.".ini")){
			$this->_lang = $lang;
			$this->_texts = parse_ini_file($this->_serverRoot."/lang/".$lang.".ini");
		}else{
			$this->_error = $this->_serverRoot."/lang/".$lang.".ini"." : Not Found (default = fr)";
		}
	}
	/**
	 * Change le thème
	 * @param string $name Le nom du thème (le nom du répertoire)
	 */
	private function setTheme($name){
		if(is_dir($this->_serverRoot."/themes/".$name)){
			$this->_theme = $name;
		}else{
			$this->_error = $this->_serverRoot."/themes/".$name." : Not Found (default = default)";
		}
	}
	/**
	 * Vérification de la validité de la page
	 * @param string $page La page demandée
	 */
	private function setPage($page){
		if($page){
			// Demande de page
			
			// La liste des pages valides
			$pages = array();
			$pages[] = "home";
			$pages[] = "login";
			$pages[] = "account";
			$pages[] = "account_update";
			$pages[] = "account_password";
			$pages[] = "account_preferences";
			$pages[] = "news";
			$pages[] = "news_archives";
			$pages[] = "news_add";
			$pages[] = "news_addpreview";
			$pages[] = "news_comments";
			$pages[] = "bookmarks";
			$pages[] = "bookmarks_groups";
			$pages[] = "bookmarks_update";
			$pages[] = "bookmarks_folderadd";
			$pages[] = "bookmarks_folderedit";
			$pages[] = "bookmarks_linkadd";
			$pages[] = "bookmarks_linkedit";
			$pages[] = "bookmarks_import";
			$pages[] = "bookmarks_importxml";
			$pages[] = "bookmarks_export";
			$pages[] = "bookmarks_exportxml";
			$pages[] = "bookmarks_check";
			$pages[] = "rss";
			$pages[] = "rss_update";
			$pages[] = "rss_archives";
			$pages[] = "rss_read";
			$pages[] = "rss_readall";
			$pages[] = "calendar";
			$pages[] = "calendar_eventadd";
			$pages[] = "calendar_eventedit";
			$pages[] = "calendar_month";
			$pages[] = "calendar_year";
			$pages[] = "calendar_todos";
			$pages[] = "calendar_todosadd";
			$pages[] = "calendar_todosedit";
			$pages[] = "contacts";
			$pages[] = "contacts_add";
			$pages[] = "contacts_view";
			$pages[] = "contacts_edit";
			$pages[] = "contacts_import";
			$pages[] = "contacts_export";
			$pages[] = "contacts_exportldif";
			$pages[] = "members";
			$pages[] = "members_details";
			$pages[] = "help";
			$pages[] = "logout";
			
			if($this->_userInfos["admin"]){
				// Les pages d'administration
				$pages[] = "news_edit";
				$pages[] = "apoadmin";
				$pages[] = "apoadmin_users";
				$pages[] = "apoadmin_groups";
				$pages[] = "apoadmin_update";
			}
			
			if(($key = array_search($page, $pages, true)) === false){
				// Par défaut, c'est la home
				$key = 0;
			}
			
			if($this->_loged){
				// S'il est loggé
				$this->_pageInfos["name"] = $pages[$key];
			}else{
				// s'il n'est pas loggé, il va à la page de login
				$this->_pageInfos["name"] = "login";
			}
		}else{
			// Aucune page demandée
			if($this->_loged){
				// S'il est loggé, il va à la home
				$this->_pageInfos["name"] = "home";
			}else{
				// s'il n'est pas loggé, il va à la page de login
				$this->_pageInfos["name"] = "login";
			}
		}
		$o = split("_", $this->_pageInfos["name"]);
		if(!isset($o[1])){
			$o[1] = "index";
		}
		
		// Le fichier d'action s'il existe
		if(file_exists($this->_serverRoot."/modules/".$o[0]."/".$o[1]."_action.inc")){
			$this->_pageInfos["action"] = $this->_serverRoot."/modules/".$o[0]."/".$o[1]."_action.inc";
		}else{
			$this->_pageInfos["action"] = false;
		}
		
		// Le fichier de script s'il existe
		if(file_exists($this->_serverRoot."/modules/".$o[0]."/".$o[1]."_script.inc")){
			$this->_pageInfos["script"] = $this->_serverRoot."/modules/".$o[0]."/".$o[1]."_script.inc";
		}else{
			$this->_pageInfos["script"] = false;
		}
		
		// Le titre de la page
		$this->_pageInfos["title"] = $this->getText("page_title_".$this->_pageInfos["name"]);
		if(!$this->_pageInfos["title"]){
			$this->_pageInfos["title"] = $this->getText("page_title_default");
		}
		
		// Enregistre le chemin de la page à afficher
		if(file_exists($this->_serverRoot."/modules/".$o[0]."/".$o[1].".inc")){
			$this->_pageInfos["path"] = $this->_serverRoot."/modules/".$o[0]."/".$o[1].".inc";
		}else{
			$this->_pageInfos["path"] = false;
			$this->_error = "Not Found : ".$this->_serverRoot."/modules/".$o[0]."/".$o[1].".inc";
		}
	}
	/*===================== FIN = METHODES PRIVEES = FIN =====================*/
	/*========================================================================*/

	/*============================ GETTER  SETTER ============================*/
	/*========================================================================*/
	/**
	 * Getters
	 */
	public function __get($var){
		switch($var){
			case "webRoot": 		return $this->_webRoot;
			case "webIndex": 		return $this->_webIndex;
			case "serverRoot": 		return $this->_serverRoot;
			case "db": 				return $this->_db;
			case "dbPrefix": 		return $this->_dbPrefix;
			case "loged": 			return $this->_loged;
			case "theme": 			return $this->_theme;
			case "lang": 			return $this->_lang;
			case "pagePath": 		return $this->_pageInfos["path"];
			case "pageTitle": 		return $this->_pageInfos["title"];
			case "pageActionPath": 	return $this->_pageInfos["action"];
			case "pageScriptPath": 	return $this->_pageInfos["script"];
			case "userId": 			return $this->_userInfos["id"];
			case "userName":		return $this->_userInfos["name"];
			case "userPassword":	return $this->_userInfos["password"];
			case "userAdmin":		return $this->_userInfos["admin"];
			case "userCreateDate":	return $this->_userInfos["createDate"];
			case "userPreferences":	return $this->_userInfos["preferences"];
			case "error": 			return (!empty($this->_error))?"<div class=\"apoError\">".$this->_error."</div>":"";
			case "message": 		return (!empty($this->_message))?"<div class=\"apoMessage\">".$this->_message."</div>":"";
			case "updateUrl": 		return $this->_updateUrl;
			case "php5Extension": 	return $this->_php5Extension;
		}
	}
	/**
	 * Setters
	 */
	public function __set($var, $value){
		switch($var){
			case "error": 			$this->_error = (string) $value; break;
			case "message": 		$this->_message = (string) $value; break;
			case "config":			$this->setConfig($value); break;
			case "lang":			$this->setLang($value); break;
			case "theme":			$this->setTheme($value); break;
			case "page":			$this->setPage($value); break;
			case "userPreferences":	$this->_userInfos["preferences"] = $value; break;
		}
	}
	/*====================== FIN = GETTER  SETTER = FIN ======================*/
	/*========================================================================*/

	/*========================== METHODES PUBLIQUES ==========================*/
	/*========================================================================*/
	/**
	 * Authentification
	 */
	public function login(){
		// Si l'utilisateur essaye de se connecter
		if($_POST["action"] == "login" && isset($_POST["login"]) && isset($_POST["password"])){
			// Récupération des données envoyées
			$getLogin = $_POST["login"];
			$getPassword = crypt($_POST["password"], "rl");
			$getPasswordNew = md5($_POST["password"]);
			$getAutoConnect = $_POST["autoconnect"];
			
			// Connexion à la base
			$this->db->select($this->_dbPrefix."users", "*", "name='".$getLogin."' AND password='".$getPassword."'");
			$row = $this->db->fetch_array();
			// Nouvelle méthode en md5
			if($row === false){
				$this->db->select($this->_dbPrefix."users", "*", "name='".$getLogin."' AND password='".$getPasswordNew."'");
				$row = $this->db->fetch_array();
			}else{
				$this->db->update($this->_dbPrefix."users", "password", "'".md5($_POST["password"])."'", "id=".$row["id"]);
				$this->db->select($this->_dbPrefix."users", "*", "name='".$getLogin."' AND password='".$getPasswordNew."'");
				$row = $this->db->fetch_array();
			}
			
			if($row !== false){
				// L'utilisateur existe dans la base de donnees
				$userId = $row["id"];
				$userName = $row["name"];
				$userPassword = $row["password"];
				
				// Creation d'une session pour cet utilisateur
				@session_start();
				$getSessionId = session_id();
				$_SESSION["userId"] = $userId;
				$_SESSION["userName"] = $userName;
				$_SESSION["userPassword"] = $userPassword;
				
				// La durée des cookies : 30 jours
				$cookieTime = 60*60*24*30;
				
				// Mode auto-connexion
				if($getAutoConnect == "on"){
					setcookie("apo_sid", $getSessionId, time() + $cookieTime);
					setcookie("apo_userid", $userId, time() + $cookieTime);
					setcookie("apo_username", $userName, time() + $cookieTime);
					setcookie("apo_userpassword", $userPassword, time() + $cookieTime);
				}else{
					setcookie("apo_sid", $getSessionId, time() + 60*5);
					setcookie("apo_userid", $userId, time() + 60*5);
					setcookie("apo_username", $userName, time() + 60*5);
					setcookie("apo_userpassword", $userPassword, time() + 60*5);
				}
				
				// L'utilisateur n'accepte pas les cookies
				if(!isset($_COOKIE["apo_sid"])){
					$this->_noCookie = true;
				}
				
				$this->_loged = true;
			}else{
				// L'utilisateur n'existe pas dans la base
				$this->_error = $this->getText("error0");
			}
		}
		
		// Authentification
		
		// Verification des cookies ou du SID
		if(isset($_COOKIE["apo_sid"])){
			// Il existe une session enregistree avec les cookies
			// On essaye de redémarrer la session serveur
			$getSid = $_COOKIE["apo_sid"];
			$getSid = preg_replace("/[^a-zA-Z0-9]*/i", "", $getSid);
			
			if(strlen($getSid) == 32){
				@session_id($getSid);
				@session_start();
			}
			
			if(!isset($_SESSION["userId"])){
				// La session serveur a expiree
				$getUserName = $_COOKIE["apo_username"];
				$getUserPassword = $_COOKIE["apo_userpassword"];
			}else{
				// La session serveur est toujours active
				$getUserName = $_SESSION["userName"];
				$getUserPassword = $_SESSION["userPassword"];
			}
			
		
		}else if(isset($_GET["sid"])){
			// La session passe par l'URL
			// On essaye de redémarrer la session serveur
			$getSid = $_GET["sid"];
			$getSid = preg_replace("/[^a-zA-Z0-9]*/i", "", $getSid);
			
			if(strlen($getSid) == 32){
				@session_id($getSid);
				@session_start();
			}
			
			if(!isset($_SESSION["userId"])){
				// La session serveur a expiree
				// Retour au login
				$this->_loged = false;
			}else{
				// La session serveur est toujours active
				$getUserName = $_SESSION["userName"];
				$getUserPassword = $_SESSION["userPassword"];
			}
			$this->_noCookie = true;
		
		
		}else{
			// Aucun cookie ni de passage de session par l'url
			// On essaye de redémarrer la session serveur
			@session_start();
			
			if(!isset($_SESSION["userId"])){
				// Aucune session enregistree
				$this->_loged = false;
			}else{
				// La session serveur est toujours active
				$getUserName = $_SESSION["userName"];
				$getUserPassword = $_SESSION["userPassword"];
			}
		}
		
		// Vérification dans la base de données
		if(!empty($getUserName) && !empty($getUserPassword) && strlen($getUserPassword) == 32){
			$this->db->select($this->_dbPrefix."users", "*", "name='".$getUserName."' AND password='".$getUserPassword."'");
			$this->_userInfos = $this->db->fetch_array();
			
			if(isset($this->_userInfos["id"]) && $this->_userInfos["id"] != ""){
				$this->_userInfos["id"] = (int) $this->_userInfos["id"];
				$this->_userInfos["admin"] = ($this->_userInfos["admin"] == "true")?true:false;
				
				// Récupération des préférences de l'utilisateur
				$this->_userInfos["preferences"] = $this->getPreferences($this->_userInfos["id"]);
				
				// Enregistrement de la date de connexion
				$this->db->update($this->_dbPrefix."users", "lastLogin", "'".gmdate("Y-m-d H:i:s")."'", "id=".$this->_userInfos["id"]);
				
				// L'utilisateur peut accéder à APO
				$this->_loged = true;
			}else{
				$this->_loged = false;
			}
		}
	}
	/**
	 * Crée un lien html
	 * @param string $page La page demandée
	 * @return string Le lien de la page demandée
	 */
	public function getLink($page){
		if($this->_noCookie){
			return $this->_webRoot."/".$page."-".session_id().".html";
		}else{
			return $this->_webRoot."/".$page.".html";
		}
	}
	/**
	 * Récupère un texte dans le fichier de langue
	 * @param string $name L'identifiant du texte dans le fichier
	 * @return string Le texte correspondant à l'identifiant, sinon false
	 */
	public function getText($name){
		if(!isset($this->_texts)){
			$this->setLang($this->_lang);
		}
		if(isset($this->_texts[$name])){
			return $this->_texts[$name];
		}else{
			$this->_error = "Text ".$name." doesn't exist";
			return false;
		}
	}
	/**
	 * Récupère les préferences d'un utilisateur
	 * @param int $userId L'ID de l'utilisateur
	 * @return array Les préférences de l'utilisateur dans un tableau
	 */
	public function getPreferences($userId){
		$this->db->select($this->_dbPrefix."users_preferences", "*", "userId=".$userId);
		$userPreferences = $this->db->fetch_array();
		if(!$userPreferences){
			// Si l'utilisateur n'a pas encore de préférences
			$this->db->replace($this->_dbPrefix."users_preferences", "userId", $userId);
			$this->db->select($this->_dbPrefix."users_preferences", "*", "userId=".$userId);
			$userPreferences = $this->db->fetch_array();
		}
		return $userPreferences;
	}
	/*==================== FIN = METHODES PUBLIQUES = FIN ====================*/
	/*========================================================================*/
}
?>