<?php
	import("com.incutio.HttpClient");
	//include_once dirname(__FILE__)."/../rss/magpie/extlib/Snoopy.class.inc";
	include_once dirname(__FILE__)."/../general/functions_html.inc";
	
	/**
	 * Fonction récursive qui génère le XML des favoris
	 * @param array $listFolders La liste des dossiers
	 * @param array $listLinks La liste des liens
	 * @param int $depth L'id courant (utilisé pour le récursif)
	 * @return string Le contenu du XML
	 */
	function bookmarks_generateXml($listFolders, $listLinks, $id=null){
		if(is_null($id)){
			$result = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?".">\n";
			$result .= "<bookmarks>\n";
			$result .= bookmarks_generateXml($listFolders, $listLinks, 0);
			$result .= "</bookmarks>";
			return $result;
		}else{
			$result = "";
			// search for sub-folders
			for ($i=0; $i<count($listFolders); $i++){
				if($listFolders[$i]["parentId"] == $id){
					$result .= "<folder id=\"".$listFolders[$i]["id"]."\" parent=\"".$listFolders[$i]["parentId"]."\" name=\"".htmlspecialchars($listFolders[$i]["name"], ENT_QUOTES, "UTF-8")."\">\n";
					$result .= bookmarks_generateXml($listFolders, $listLinks, $listFolders[$i]["id"]);
					$result .= "</folder>\n";
				}
			}
			//search for sub-links
			for ($i=0; $i<count($listLinks); $i++){
				if($listLinks[$i]["parentId"] == $id){
					$result .= "<link id=\"".$listLinks[$i]["id"]."\" folder=\"".$listLinks[$i]["parentId"]."\" name=\"".htmlspecialchars($listLinks[$i]["name"], ENT_QUOTES, "UTF-8")."\" url=\"".htmlspecialchars($listLinks[$i]["url"], ENT_QUOTES, "UTF-8")."\" />\n";
				}
			}
			return $result;
		}
	}
	/**
	 * Parse un fichier XML et retourne un affichage html
	 * @param string $file L'url du fichier xml à parser
	 * @return string Le contenu du html
	 */
	function bookmarks_parseXML($file){
		global $apo, $bookmarks_depth, $bookmarks_nbElement, $bookmarks_content;
		$bookmarks_depth = array();
		$bookmarks_nbElement = 0;
		$bookmarks_content = "<dl id=\"bookmarksList\">";
		
		$xml_parser = xml_parser_create("UTF-8");
		xml_set_element_handler($xml_parser, "bookmarks_parseXML_beginElement", "bookmarks_parseXML_endElement");
		if (!($file_ressource = @fopen($file, "r"))) {
			// Impossible d'ouvrir le fichier XML
			$apo->error = $apo->getText("error7")." : ".$file;
		}else{
			// On parse le XML
			while ($data = fread($file_ressource, 4096)) {
			    if (!xml_parse($xml_parser, $data, feof($file_ressource))) {
			        die(sprintf("Affichage HTML > erreur XML : %s à la ligne %d",
			                    xml_error_string(xml_get_error_code($xml_parser)),
			                    xml_get_current_line_number($xml_parser)));
			    }
			}
			xml_parser_free($xml_parser);
		}
		$bookmarks_content .= "</dl>";
		return $bookmarks_content;
	}
	
	function bookmarks_parseXML_beginElement($parser, $name, $attrs) {
		global $apo, $bookmarks_depth, $bookmarks_nbElement, $bookmarks_content;

		$bookmarks_nbElement++;
		if($name == "FOLDER"){
			list($attr, $idValue) = each($attrs);
			list($attr, $parentValue) = each($attrs);
			list($attr, $nameValue) = each($attrs);
			
			$bookmarks_content .= "<dt>";
			$bookmarks_content .= "<a class=\"folder\" href=\"javascript:switchDisplay('bookmarksFolder".$bookmarks_nbElement."')\">\n";
			// Ajouter de l'espace avant en fonction de la profondeur
			if($bookmarks_depth[$parser] > 1){
				$bookmarks_content .= str_repeat("&nbsp;&nbsp;&nbsp;&nbsp;", $bookmarks_depth[$parser] - 1);
			}
			// L'icone du dossier
			$bookmarks_content .= "<img src=\"".$apo->webRoot."/themes/".$apo->theme."/images/bookmarks/folder.gif\" alt=\"\" />&nbsp;";
			
			$bookmarks_content .= utf2html(htmlentities(html_entity_decode($nameValue, ENT_QUOTES, "UTF-8"), ENT_QUOTES, "UTF-8"));
			$bookmarks_content .= "</a>\n";
			$bookmarks_content .= "</dt>";
			$bookmarks_content .= "<dd id=\"bookmarksFolder".$bookmarks_nbElement."\" class=\"bookmarksFolder\">";
			$bookmarks_content .= "<dl>";
		}elseif($name == "LINK"){
			list($attr, $attrId) = each($attrs);
			list($attr, $attrFolderId) = each($attrs);
			list($attr, $attrName) = each($attrs);
			list($attr, $attrURL) = each($attrs);
			
			$bookmarks_content .= "<dt>";
			$bookmarks_content .= '<a class="url" href="'.utf2html(htmlentities(html_entity_decode($attrURL, ENT_QUOTES, "UTF-8"), ENT_QUOTES, "UTF-8")).'"';
			if(!isset($apo->userPreferences["bookmarks_link_popup"]) || $apo->userPreferences["bookmarks_link_popup"] == "1"){
				$bookmarks_content .= ' onclick="window.open(this.href);return false;"';
			}
			$bookmarks_content .= ">\n";
			// Ajouter de l'espace avant en fonction de la profondeur
			$bookmarks_content .= str_repeat("&nbsp;&nbsp;&nbsp;&nbsp;", $bookmarks_depth[$parser] - 1);
			// L'icone du lien
			$bookmarks_content .= '<img src="'.$apo->webRoot."/themes/".$apo->theme.'/images/bookmarks/link.gif" alt="" />&nbsp;'.utf2html(htmlentities(html_entity_decode($attrName, ENT_QUOTES, "UTF-8"), ENT_QUOTES, "UTF-8"))."</a>\n";
			$bookmarks_content .= "</dt>";
		}
    	$bookmarks_depth[$parser]++;
	}

	function bookmarks_parseXML_endElement($parser, $name) {
		global $bookmarks_depth, $bookmarks_nbElement, $bookmarks_content;
		if($name == "FOLDER"){
			$bookmarks_content .= "<dt></dt></dl></dd>";
		}elseif($name == "LINK"){
			
		}
		$bookmarks_depth[$parser]--;
	}
	/**
	 * Parse un fichier XML et retourne un affichage html admin
	 * @param string $file L'url du fichier xml à parser
	 * @return string Le contenu du html admin
	 */
	function bookmarks_parseXMLAdmin($file, $type="normal"){
		global $apo, $bookmarks_depth, $bookmarks_nbElement, $bookmarks_content, $bookmarks_contentType;
		$bookmarks_depth = array();
		$bookmarks_nbElement = 0;
		$bookmarks_contentType = $type;
		$bookmarks_content = "<dl id=\"bookmarksList\">";
		
		$xml_parser = xml_parser_create("UTF-8");
		xml_set_element_handler($xml_parser, "bookmarks_parseXMLAdmin_beginElement", "bookmarks_parseXMLAdmin_endElement");
		if (!($file_ressource = @fopen($file, "r"))) {
			// Impossible d'ouvrir le fichier XML
		    $apo->error = $apo->getText("error7")." : ".$file;
		}else{
			// On parse le XML
			while ($data = fread($file_ressource, 4096)) {
			    if (!xml_parse($xml_parser, $data, feof($file_ressource))) {
			        die(sprintf("Affichage HTML Admin > erreur XML : %s à la ligne %d",
			                    xml_error_string(xml_get_error_code($xml_parser)),
			                    xml_get_current_line_number($xml_parser)));
			    }
			}
			xml_parser_free($xml_parser);
		}
		$bookmarks_content .= "</dl>";
		return $bookmarks_content;
	}
	
	function bookmarks_parseXMLAdmin_beginElement($parser, $name, $attrs) {
		global $apo, $bookmarks_depth, $bookmarks_nbElement, $bookmarks_content, $bookmarks_contentType;

		$bookmarks_nbElement++;
		if($name == "FOLDER"){
			list($attr, $idValue) = each($attrs);
			list($attr, $parentValue) = each($attrs);
			list($attr, $nameValue) = each($attrs);
			
			$bookmarks_content .= "<dt class=\"folder\">";
				// Dossier
				if($bookmarks_contentType == "group"){
					$bookmarks_content .= "<input type=\"radio\" name=\"targetid\" value=\"".$idValue."\" />";
				}
				$bookmarks_content .= "<a class=\"adminfolder\" href=\"javascript:switchDisplay('bookmarksFolder".$bookmarks_nbElement."')\">\n";
				// Ajouter de l'espace avant en fonction de la profondeur
				$bookmarks_content .= str_repeat("&nbsp;&nbsp;&nbsp;&nbsp;", $bookmarks_depth[$parser] - 1);
				// L'icone du dossier
				$bookmarks_content .= "<img src=\"".$apo->webRoot."/themes/".$apo->theme."/images/bookmarks/folder.gif\" alt=\"\" />&nbsp;";
				$bookmarks_content .= utf2html(htmlentities(html_entity_decode($nameValue, ENT_QUOTES, "UTF-8"), ENT_QUOTES, "UTF-8"));
				$bookmarks_content .= "</a>";
				// Options
				if($bookmarks_contentType == "normal"){	
					$bookmarks_content .= " <a class=\"folderEdit\" href=\"".$apo->getLink("bookmarks_folderedit".$idValue)."\">".$apo->getText("button_edit")."</a>\n";
				}
			$bookmarks_content .= "</dt>";
			$bookmarks_content .= "<dd id=\"bookmarksFolder".$bookmarks_nbElement."\" class=\"bookmarksFolder\">\n";
			$bookmarks_content .= "<dl>";
		}elseif($name == "LINK"){
			list($attr, $attrId) = each($attrs);
			list($attr, $attrFolderId) = each($attrs);
			list($attr, $attrName) = each($attrs);
			list($attr, $attrURL) = each($attrs);
			
			$bookmarks_content .= "<dt class=\"bookmarksLink\">\n";
			if($bookmarks_contentType == "group"){
				$bookmarks_content .= "<input type=\"radio\" name=\"targetid\" value=\"".$attrId."\" />";
			}
			$bookmarks_content .= "<a class=\"adminurl\" href=\"".utf2html(htmlentities(html_entity_decode($attrURL, ENT_QUOTES, "UTF-8"), ENT_QUOTES, "UTF-8"))."\" onclick=\"window.open(this.href);return false;\">";
			// Ajouter de l'espace avant en fonction de la profondeur
			$bookmarks_content .= str_repeat("&nbsp;&nbsp;&nbsp;&nbsp;", $bookmarks_depth[$parser] - 1);
			// L'icone du lien
			$bookmarks_content .= "<img src=\"".$apo->webRoot."/themes/".$apo->theme."/images/bookmarks/link.gif\" alt=\"\" />&nbsp;".utf2html(htmlentities(html_entity_decode($attrName, ENT_QUOTES, "UTF-8"), ENT_QUOTES, "UTF-8"))."</a>";
			// Options
			if($bookmarks_contentType == "normal"){
				$bookmarks_content .= " <a class=\"linkEdit\" href=\"".$apo->getLink("bookmarks_linkedit".$attrId)."\">".$apo->getText("button_edit")."</a>\n";
			}
			$bookmarks_content .= "</dt>";
		}
    	$bookmarks_depth[$parser]++;
	}

	function bookmarks_parseXMLAdmin_endElement($parser, $name) {
		global $bookmarks_depth, $bookmarks_nbElement, $bookmarks_content, $bookmarks_contentType;
		if($name == "FOLDER"){
			$bookmarks_content .= "<dt></dt></dl></dd>";
		}elseif($name == "LINK"){
			
		}
		$bookmarks_depth[$parser]--;
	}
	/**
	 * Génère les <option> pour affichage <select>
	 * @param string $file L'url du fichier xml à parser
	 * @return string Le contenu du <select>
	 */
	function bookmarks_generateSelectFolder($file){
		global $apo, $bookmarks_depth, $bookmarks_nbElement, $bookmarks_content;
		$bookmarks_depth = array();
		$bookmarks_nbElement = 0;
		$bookmarks_content = "";
		
		$xml_parser = xml_parser_create("UTF-8");
		xml_set_element_handler($xml_parser, "bookmarks_generateSelectFolder_beginElement", "bookmarks_generateSelectFolder_endElement");
		if (!($file_ressource = @fopen($file, "r"))) {
			// Impossible d'ouvrir le fichier XML
		    $apo->error = $apo->getText("error7")." : ".$file;
		}else{
			// On parse le XML
			while ($data = fread($file_ressource, 4096)) {
			    if (!xml_parse($xml_parser, $data, feof($file_ressource))) {
			        die(sprintf("erreur XML : %s à la ligne %d",
			                    xml_error_string(xml_get_error_code($xml_parser)),
			                    xml_get_current_line_number($xml_parser)));
			    }
			}
			xml_parser_free($xml_parser);
		}
		
		return $bookmarks_content;
	}
	
	function bookmarks_generateSelectFolder_beginElement($parser, $name, $attrs) {
		global $bookmarks_depth, $bookmarks_nbElement, $bookmarks_content;

		$bookmarks_nbElement++;
		if($name == "FOLDER"){
			list($attr, $idValue) = each($attrs);
			list($attr, $parentValue) = each($attrs);
			list($attr, $nameValue) = each($attrs);
			
			$bookmarks_content .= "<option value=\"".$idValue."\">";
			// Ajouter de l'espace avant en fonction de la profondeur
			
			if($bookmarks_depth[$parser] > 1){
				$bookmarks_content .= str_repeat("&nbsp;&nbsp;", $bookmarks_depth[$parser]);
				$bookmarks_content .= "+";
			}
			$bookmarks_content .= utf2html(htmlentities(html_entity_decode($nameValue, ENT_QUOTES, "UTF-8"), ENT_QUOTES, "UTF-8"));
			$bookmarks_content .= "</option>\n";
		}
    	$bookmarks_depth[$parser]++;
	}

	function bookmarks_generateSelectFolder_endElement($parser, $name) {
		global $bookmarks_depth, $bookmarks_nbElement, $bookmarks_content;
		$bookmarks_depth[$parser]--;
	}
	/**
	 * Ajoute des favoris xml dans la base
	 * @param SimpleXMLElement $xmlnod Un objet SimpleXMLElement
	 * @param int $lastindex Le dernier index de la table
	 * @param int $idparent L'ID du parent (dans la hiérarchie des favoris
	 * @return string La grande requête qui va tout ajouter
	 */
	function addXml(&$xmlnod, &$lastindex, $idparent=0, &$tbresult=array()){
		global $apo;
		
		foreach ($xmlnod->children() as $childnod) {
			$name = $childnod["name"];
			$url = $childnod["url"];
			if(isset($childnod["url"])){
				// C'est une URL, ca s'arrête là
				$tbresult[] = array($apo->userId, $idparent, "'link'", "'".$name."'", "'".$url."'");
				$lastindex++;
			}else{
				// C'est un dossier
				$tbresult[] = array($apo->userId, $idparent, "'folder'", "'".$name."'", "''");
				$lastindex++;
				addXml($childnod, $lastindex, $lastindex, $tbresult);
			}
		}
		return $tbresult;
	}
	/**
	 * Check le lien et retourne le status
	 * @param string $url L'url du lien
	 * @return array Le status du lien $result["type"], $result["status"] et $result["message"]
	 */
	function link_check($url, $maxRedirect=5){
		$a = parse_url($url);
		$host = $a["host"];
		$path = ($a["path"] == "" || !isset($a["path"]))?"/":$a["path"];
		$o = new HttpClient($host);
		$o->setHeadersOnly(true);
		$o->setMaxRedirects($maxRedirect);
		//$o = new Snoopy;
		
		$result = array();
		
		if (!$o->get($path)) {
		//if (!$o->fetch($url)) {
			$result["type"] = "x";
			$result["status"] = "xxx";
			$result["message"] = "An error occurred";
		}else{
			$status = $o->getStatus();
			//$status = $o->$status;
			$result["status"] = $status;
			$s = substr($status, 0, 1);
			$result["type"] = $s;
			//Successful 2xx
			//Redirection 3xx
			//Client Error 4xx
			//Server Error 5xx
			switch($status){					
				case "200": $result["message"] = "OK"; break;
				case "201": $result["message"] = "Created"; break;
				case "202": $result["message"] = "Accepted"; break;
				case "203": $result["message"] = "Non-Authoritative Information"; break;
				case "204": $result["message"] = "No Content"; break;
				case "205": $result["message"] = "Reset Content"; break;
				case "206": $result["message"] = "Partial Content"; break;
				
				case "300": $result["message"] = "Multiple Choices"; break;
				case "301": $result["message"] = "Moved Permanently"; break;
				case "302": $result["message"] = "Found"; break;
				case "303": $result["message"] = "See Other"; break;
				case "304": $result["message"] = "Not Modified"; break;
				case "305": $result["message"] = "Use Proxy"; break;
				case "307": $result["message"] = "Temporary Redirect"; break;
				
				case "400": $result["message"] = "Bad Request"; break;
				case "401": $result["message"] = "Unauthorized"; break;
				case "402": $result["message"] = "Payment Required"; break;
				case "403": $result["message"] = "Forbidden"; break;
				case "404": $result["message"] = "Not Found"; break;
				case "405": $result["message"] = "Method Not Allowed"; break;
				case "406": $result["message"] = "Not Acceptable"; break;
				case "407": $result["message"] = "Proxy Authentication Required"; break;
				case "408": $result["message"] = "Request Timeout"; break;
				case "409": $result["message"] = "Conflict"; break;
				case "410": $result["message"] = "Gone"; break;
				case "411": $result["message"] = "Length Required"; break;
				case "412": $result["message"] = "Precondition Failed"; break;
				case "413": $result["message"] = "Request Entity Too Large"; break;
				case "414": $result["message"] = "Request-URI Too Long"; break;
				case "415": $result["message"] = "Unsupported Media Type"; break;
				case "416": $result["message"] = "Requested Range Not Satisfiable"; break;
				case "417": $result["message"] = "Expectation Failed"; break;
				
				case "500": $result["message"] = "Internal Server Error"; break;
				case "501": $result["message"] = "Not Implemented"; break;
				case "502": $result["message"] = "Bad Gateway"; break;
				case "503": $result["message"] = "Service Unavailable"; break;
				case "504": $result["message"] = "Gateway Timeout"; break;
				case "505": $result["message"] = "HTTP Version Not Supported"; break;
				default : $result["message"] = "Unknown";
			}
		}
		return $result;
	}
?>